
// This program reads the 9 "Degrees of Freedom" (DOF) devices on the STM32L476 Discovery board
// This involves reading acceleration and magnetic field from the LSM303 and angular velocity from the L3DG20 for the Z Axis
// Change to suit whichever axis you prefer.
// The results of these measurements are output to a serial terminal.  They are quite noisy and may need filtering
// depending on your application.  
// The program also includes a SysTick timer interrupt which may be useful to you if you want to perform some readings at a 
// predictable rate.
#include "stm32l476.h"
#include <math.h>
#include "serial.h"
#include "lsm303dlhc.h"
#include "spi2.h"
#include "l3gd20.h"
// RED LED on PB2 
// GREEN LED ON PE8
void DoDataAcquisition(void);
void delay(unsigned dly)
{
    while(dly--);
}
void initClocks()
{
	// Initialize the clock system to a higher speed.
	// At boot time, the clock is derived from the MSI clock 
	// which defaults to 4MHz.  Will set it to 80MHz
	// See chapter 6 of the reference manual (Document DM00083560)
	RCC_CR &= ~BIT24; // Make sure PLL is off
	
	// PLL Input clock = MSI so BIT1 = 1, BIT 0 = 0
	// PLLM = Divisor for input clock : set = 1 so BIT6,5,4 = 0
	// PLL-VCO speed = PLL_N x PLL Input clock
	// This must be < 344MHz
	// PLL Input clock = 4MHz from MSI
	// PLL_N can range from 8 to 86.  
	// Will use 86 for PLL_N as 80 * 4 = 320MHz
	// Put value 86 into bits 14:8 (being sure to clear bits as necessary)
	// PLLSAI3 : Serial audio interface : not using leave BIT16 = 0
	// PLLP : Must pick a value that divides 320MHz down to <= 80MHz
	// If BIT17 = 1 then divisor is 17; 320/17 = 18.82MHz : ok
	// PLLQEN : Don't need this so set BIT20 = 0
	// PLLQ : Must divide 320 down to value <=80MHz.  
	// Set BIT22,21 to 1 to get a divisor of 8 : ok
	// PLLREN : This enables the PLLCLK output of the PLL
	// I think we need this so set to 1. BIT24 = 1 
	// PLLR : Pick a value that divides 320 down to <= 80MHz
	// Choose 4 to give an 80MHz output.  
	// BIT26 = 0; BIT25 = 1
	// All other bits reserved and zero at reset
	RCC_PLLCFGR = BIT25 + BIT24 + BIT22 + BIT21 + \
				  BIT17 + (80 << 8) + BIT0;	
	RCC_CR |= BIT24; // Turn PLL on
	while( (RCC_CR & BIT25)== 0); // Wait for PLL to be ready
	// configure flash for 4 wait states (required at 80MHz)
	FLASH_ACR &= ~(BIT2+BIT1+BIT0);
	FLASH_ACR |= BIT2; 
	RCC_CFGR |= BIT1+BIT0; // Select PLL as system clock
}
int ButtonPressed()
{
	// Check to see if a button is pressed
	// Joystick is on PA0,1,2,3 and 5
	// Will check for PA0 - the centre button
	if (GPIOA_IDR & BIT0)
		return 1;
	else
		return 0;
}
void configPins()
{
    // Turn on GPIOA, GPIOB, GPIOC and GPIOE
    RCC_AHB2ENR |= BIT0 + BIT1 + BIT2 + BIT4;
    GPIOA_MODER &= ~(BIT0 + BIT1); // PA0 is an input
    GPIOA_PUPDR &= ~BIT0;
    GPIOA_PUPDR |= BIT1;  // Turn on pull down resistor for PA0
    GPIOB_MODER |= BIT4;
    GPIOB_MODER &= ~BIT5;
    GPIOC_MODER |= BIT0;  // Make PC0 an output for the Compass interface
    GPIOC_MODER &= ~BIT1;
    GPIOE_MODER |= BIT16; // Make PE0 an output for the Acceleromenter interface
    GPIOE_MODER &= ~BIT17;
    
	
}
void initSysTick()
{
	
	// The systick timer is driven by an 80MHz clock
	// Divide this down to achieve a 1ms timebase
	// Divisor = 48MHz/1000Hz
	// Reload value = 48000-1
	// enable systick and its interrupts
	STK_CSR |=(BIT0+BIT1+BIT2); 
	STK_RVR=80000-1; // generate 1 millisecond time base
	STK_CVR=5;
	enable_interrupts();
}
uint32_t msCounter = 0;
void SysTick()
{
	// SysTick interrupt service routine
	static int State = 0;
	msCounter++;
	if (msCounter >= 1000)
	{
                DoDataAcquisition();
		msCounter = 0;
		if (State == 0)
		{
			GPIOE_ODR &= ~BIT8; // Turn off green led
			State = 1;
		}
		else
		{
			GPIOE_ODR |= BIT8;  // Turn on green led
			State = 0;
		}
	}
}

extern int DebugArray[10];
int main()
{
    int TestReading;
    
    int Index;
    // Don't go any further mucking about with clocks 
    // before user input or could brick the board
    configPins();
    while(!ButtonPressed());
    initUART(9600);
    eputs("80MHz!! Wheeeeee!\r\n");	
    initClocks();
    initSysTick();
    initSPI2();
    initAccel();
    initCompass();
    initGyro();

 

    while(1)
    {
        GPIOB_ODR |= BIT2;  // Turn on red led        
        delay(200000);
        GPIOB_ODR &= ~BIT2; // Turn off red led        
        //DoDataAcquisition();
        delay(1000000);
    }
}
void DoDataAcquisition(void)
{

    short GyroReading;
    short CompassReading;
    short AcceleromenterReading;
    GyroReading = (ReadGyroRegister(0x2d) << 8)+ReadGyroRegister(0x2c); // connection to Accel ok?		
    AcceleromenterReading =  (ReadAccelRegister(0x2d) << 8)+ReadAccelRegister(0x2c);
    CompassReading =  (ReadCompassRegister(0x2d) << 8)+ReadCompassRegister(0x2c);        
    eputLong(AcceleromenterReading);
    eputs(" ");    
    eputLong(CompassReading);
    eputs(" ");           
    eputLong(GyroReading);
    eputs("\r\n");		    
}